import sys
import argparse
from math import *

import Bio.PDB
from Bio import pairwise2

import numpy as np

import matplotlib
matplotlib.use('Agg')
from matplotlib.backends.backend_pdf import PdfPages
import matplotlib.pyplot as plt
import matplotlib.cm as cm

from parsing import parse_contacts
from parsing import parse_psipred
from parsing import parse_fasta
from parsing import parse_pdb


def get_min_dist(res1, res2):
    
    min_dist = float('inf')

    for atm1 in res1:
        for atm2 in res2:
            diff_vec = atm1 - atm2
            dist = np.sqrt(np.sum(diff_vec * diff_vec))
            if dist < min_dist:
                min_dist = dist

    return min_dist


def get_heavy_contacts(gapped_res_lst):

    seqlen = len(gapped_res_lst)
    dist_mat = np.zeros((seqlen, seqlen), np.float)
    dist_mat.fill(float('inf'))
    
    for i, res1 in enumerate(gapped_res_lst):
        if res1 == '-':
            continue
        for j, res2 in enumerate(gapped_res_lst):
            if res2 == '-':
                continue
            dist_mat[i,j] = get_min_dist(res1[1], res2[1])
    return dist_mat


def get_cb_contacts(gapped_cb_lst):

    seqlen = len(gapped_cb_lst)
    dist_mat = np.zeros((seqlen, seqlen), np.float)
    dist_mat.fill(float('inf'))
    
    for i, cb1 in enumerate(gapped_cb_lst):
        if cb1 == '-':
            continue
        for j, cb2 in enumerate(gapped_cb_lst):
            if cb2 == '-':
                continue
            diff_vec = cb1 - cb2
            dist_mat[i,j] = np.sqrt(np.sum(diff_vec * diff_vec))
    return dist_mat



def get_ppvs(contacts_x, contacts_y, ref_contact_map, atom_seq_ali, ref_len, factor):

    PPVs = []
    TPs = []
    FPs = []

    for num_c in range(min(len(contacts_x), ref_len * factor))[1:]:
        TP = 0.0
        FP = 0.0
        for i in range(num_c):
            c_x = contacts_x[i]
            c_y = contacts_y[i]
            if atom_seq_ali[c_x] == '-':
                continue
            if atom_seq_ali[c_y] == '-':
                continue
            if ref_contact_map[c_x, c_y] > 0:
                TP += 1.0
            else:
                FP += 1.0

        if TP > 0 and FP > 0:
            PPVs.append(TP / (TP + FP))
            TPs.append(TP / ref_len)
            FPs.append(FP / ref_len)

    if len(PPVs) == 0:
        PPVs.append(0.0)

    return PPVs


def get_tp_colors(contacts_x, contacts_y, ref_contact_map, atom_seq_ali):

    tp_colors = []

    for i in range(len(contacts_x)):
        c_x = contacts_x[i]
        c_y = contacts_y[i]
        if atom_seq_ali[c_x] == '-':
            #tp_colors.append('green')
            tp_colors.append('red')
            continue
        if atom_seq_ali[c_y] == '-':
            #tp_colors.append('green')
            tp_colors.append('red')
            continue
        if ref_contact_map[c_x, c_y] > 0:
            tp_colors.append('blue')
        else:
            tp_colors.append('red')

    return tp_colors
 

def plot_map(fasta_filename, c_filename, factor, c2_filename='', psipred_filename='', pdb_filename='', is_heavy=False, chain='', sep=',', outfilename=''):  
   
    acc = c_filename.split('.')[0]

    ### get sequence
    seq = parse_fasta.read_fasta(open(fasta_filename, 'r')).values()[0][0]
    ref_len = len(seq)

    ### get top "factor" * "ref_len" predicted contacts
    contacts = parse_contacts.parse(open(c_filename, 'r'), sep)

    contacts_x = []
    contacts_y = []
    scores = []
    contact_dict = {}

    count = 0
    for i in range(len(contacts)):
        score = contacts[i][0]
        c_x = contacts[i][1] - 1
        c_y = contacts[i][2] - 1

        pos_diff = abs(c_x - c_y)
        too_close = pos_diff < 5

        if not too_close:
            contacts_x.append(c_x)
            contacts_y.append(c_y)
            scores.append(score)
            count += 1
           
        if count >= ref_len * factor:
            break
 

    ### start plotting
    fig = plt.figure()
    ax = fig.add_subplot(111)

    ### plot secondary structure on the diagonal if given
    if psipred_filename:
        ss = parse_psipred.horizontal(open(psipred_filename, 'r'))
        for i in range(len(ss)):
            if ss[i] == 'H':
                plt.plot(i, i, 'o', c='#8B0043', mec="#8B0043", markersize=2)
            if ss[i] == 'E':
                plt.plot(i, i, 'D', c='#0080AD', mec="#0080AD", markersize=2)
            if ss[i] == 'C':
                continue

    ### plot reference contacts in the background if given
    if pdb_filename:
        res_lst = parse_pdb.get_coordinates(open(pdb_filename, 'r'), chain)
        cb_lst = parse_pdb.get_cb_coordinates(open(pdb_filename, 'r'), chain)
        atom_seq = parse_pdb.get_atom_seq(open(pdb_filename, 'r'), chain)

        align = pairwise2.align.globalms(atom_seq, seq, 2, -1, -0.5, -0.1)

        atom_seq_ali = align[-1][0]
        seq_ali = align[-1][1]

        j = 0
        gapped_res_lst = []
        gapped_cb_lst = []

        for i in xrange(len(atom_seq_ali)):
            if atom_seq_ali[i] == '-':
                gapped_res_lst.append('-')
                gapped_cb_lst.append('-')
            elif seq_ali[i] == '-':
                j += 1
                continue
            else:
                gapped_res_lst.append(res_lst[j])
                gapped_cb_lst.append(cb_lst[j])
                j += 1

        if is_heavy:
            dist_mat = get_heavy_contacts(gapped_res_lst)
            heavy_cutoff = 5
            ref_contact_map = dist_mat < heavy_cutoff
            ref_contacts = np.where(dist_mat < heavy_cutoff)
        else:
            dist_mat = get_cb_contacts(gapped_cb_lst)
            cb_cutoff = 8
            ref_contact_map = dist_mat < cb_cutoff
            ref_contacts = np.where(dist_mat < cb_cutoff)
        
        ref_contacts_x = ref_contacts[0]
        ref_contacts_y = ref_contacts[1]
       
        PPVs = get_ppvs(contacts_x, contacts_y, ref_contact_map, atom_seq_ali, ref_len, factor)
        tp_colors = get_tp_colors(contacts_x, contacts_y, ref_contact_map, atom_seq_ali)
   
        print '%s\t%s' % (acc, PPVs[-1])
      
        ax.scatter(ref_contacts_x, ref_contacts_y, marker='o', c='#CCCCCC', lw=0)


    ### plot predicted contacts from second contact map if given
    if c2_filename:
        contacts2 = parse_contacts.parse(open(c2_filename, 'r'), sep)
        contacts2_x = []
        contacts2_y = []
        scores2 = []
        contact_dict2 = {}

        count = 0

        for i in range(len(contacts2)):
            score = contacts2[i][0]
            c_x = contacts2[i][1] - 1
            c_y = contacts2[i][2] - 1

            pos_diff = abs(c_x - c_y)
            too_close = pos_diff < 5

            if not too_close:
                contacts2_x.append(c_x)
                contacts2_y.append(c_y)
                scores2.append(score)
                count += 1
               
            if count >= ref_len * factor:
                break

        ### use TP/FP color coding if reference contacts given
        if pdb_filename:
            PPVs2 = get_ppvs(contacts2_x, contacts2_y, ref_contact_map, atom_seq_ali, ref_len, factor)
            tp2_colors = get_tp_colors(contacts2_x, contacts2_y, ref_contact_map, atom_seq_ali)
            print '%s\t%s' % (acc, PPVs2[-1])
            fig.suptitle('%s\nPPV (upper left) = %.2f | PPV (lower right) = %.2f' % (acc, PPVs[-1], PPVs2[-1]))
            sc = ax.scatter(contacts2_y[::-1], contacts2_x[::-1], marker='o', c=tp2_colors[::-1], s=6, alpha=0.75, linewidths=0.0)
            sc = ax.scatter(contacts_x[::-1], contacts_y[::-1], marker='o', c=tp_colors[::-1], s=6, alpha=0.75, linewidths=0.0)
        else:
            sc = ax.scatter(contacts2_y[::-1], contacts2_x[::-1], marker='o', c='#D70909', edgecolor='#D70909', s=4, linewidths=0.5)
            sc = ax.scatter(contacts_x[::-1], contacts_y[::-1], marker='o', c='#004F9D', edgecolor='#004F9D', s=4, linewidths=0.5)


    ### plot predicted contacts from first contact map on both triangles
    ### if no second contact map given
    else:
        if pdb_filename:
            fig.suptitle('%s\nPPV = %.2f' % (acc, PPVs[-1]))
            sc = ax.scatter(contacts_x[::-1], contacts_y[::-1], marker='o', c=tp_colors[::-1], s=6, alpha=0.75, linewidths=0.0)
            sc = ax.scatter(contacts_y[::-1], contacts_x[::-1], marker='o', c=tp_colors[::-1], s=6, alpha=0.75, linewidths=0.0)
        else:
            sc = ax.scatter(contacts_x[::-1], contacts_y[::-1], marker='o', c=scores[::-1], s=4, alpha=0.75, cmap=cm.jet, linewidths=0.1)
            sc = ax.scatter(contacts_y[::-1], contacts_x[::-1], marker='o', c=scores[::-1], s=4, alpha=0.75, cmap=cm.jet, linewidths=0.1)
            plt.colorbar(sc)

    plt.gca().set_xlim([0,ref_len])
    plt.gca().set_ylim([0,ref_len])

    if outfilename:
        if outfilename.endswith('.pdf'):
            pp = PdfPages(outfilename)
            pp.savefig(fig)
            pp.close()
        else:
            pp = PdfPages('%s.pdf' % outfilename)
            pp.savefig(fig)
            pp.close()
    else:
        pp = PdfPages('%s_ContactMap.pdf' % c_filename)
        pp.savefig(fig)
        pp.close()


    
if __name__ == "__main__":

    p = argparse.ArgumentParser(description='Plot protein residue contact maps.')
    p.add_argument('fasta_file')#, required=True)
    p.add_argument('contact_file')#, required=True)
    p.add_argument('-o', '--outfile', default='')
    p.add_argument('-f', '--factor', default=2.0, type=float)
    p.add_argument('--c2', default='')
    p.add_argument('--psipred_horiz', default='')
    p.add_argument('--pdb', default='')
    p.add_argument('--heavy', action='store_true')
    p.add_argument('--chain', default='')

    args = vars(p.parse_args(sys.argv[1:]))

    fasta_filename = args['fasta_file']
    c_filename = args['contact_file']
    psipred_filename = args['psipred_horiz']

    # guessing separator of constraint file
    line = open(c_filename,'r').readline()
    if len(line.split(',')) != 1:
        sep = ','
    elif len(line.split(' ')) != 1:
        sep = ' '
    else:
        sep = '\t'
    
    plot_map(args['fasta_file'], args['contact_file'], args['factor'], c2_filename=args['c2'], psipred_filename=args['psipred_horiz'], pdb_filename=args['pdb'], is_heavy=args['heavy'], chain=args['chain'], sep=sep, outfilename=args['outfile'])




#!/usr/bin/env python


def parse(afile, sep=' '):
    
    """Parse contact file.
    @param  afile   contact file
    @param  sep     separator of contact file (default=' ')
    Ensures: Output is sorted by confidence score.
    @return [(score, residue a, residue b)]
    """

    contacts = []
    for aline in afile:
        if aline.strip() != '':
            line_arr = aline.strip().split(sep)
            if line_arr[0].startswith('E'):
                continue
            i = int(line_arr[0])
            j = int(line_arr[1])
            score = float(line_arr[-1])
            if abs(i - j) > 4:
                contacts.append((score, i, j))
    afile.close()

    contacts.sort(key=lambda x: x[0], reverse=True)
    return contacts



#!/usr/bin/env python


def horizontal(hfile):

    """Reads psipred output .horiz file.
    @param  hfile   psipred .horiz file
    @return secondary structure string.
    """
    
    result = ''
    for line in hfile:
        line_arr = line.strip().split(' ')
        if line_arr[0] == 'Pred:' and len(line_arr) > 1:
            result += line_arr[1]
    return result


def horizontal_conf(hfile):

    """Reads psipred output .horiz file.
    @param  hfile   psipred .horiz file
    @return secondary structure string.
    """
    
    result = ''
    for line in hfile:
        line_arr = line.strip().split(' ')
        if line_arr[0] == 'Conf:' and len(line_arr) > 1:
            result += line_arr[1]
    return result


def horizontal_seq(hfile):

    """Reads psipred output .horiz file.
    @param  hfile   psipred .horiz file
    @return amino acid sequence.
    """

    result = ''
    for line in hfile:
        line_arr = line.strip().split(' ')
        if line_arr[0] == 'AA:':
            result += line_arr[1]
    return result



#!/usr/bin/env python

import string, copy
import sys

def read_fasta(afile, query_id=''):

    """Parses any fasta, a2m, a3m file, sequence or alignment file.
    @param  afile       input file
    @param  query_id    ID of query sequence (default='')
    Ensures: key of a given query ID only contains its ID, not the full header
    @return {header: [sequence_1, sequence_2, ...]} 
    """

    seq_dict = {}
    header = ''
    seq = ''

    for aline in afile:
        aline = aline.strip()

        # check for header
        if aline.startswith('>'):
            if header != '' and seq != '':
                if seq_dict.has_key(header):
                    seq_dict[header].append(seq)
                else:
                    seq_dict[header] = [seq]
            seq = ''
            if aline.startswith('>%s' % query_id) and query_id !='':
                header = query_id
            else:
                header = aline[1:]

        # otherwise concatenate sequence
        else:
            #aline_seq = aline.translate(None, '.-').upper()
            seq += aline

    # add last entry
    if header != '':
        if seq_dict.has_key(header):
            seq_dict[header].append(seq)
        else:
            seq_dict[header] = [seq]
    else:
        sys.stderr.write('ERROR: file empty or wrong file format')

    return seq_dict


def read_fasta_pdb(afile, query_id=''):

    """Parses any fasta, a2m, a3m file, sequence or alignment file.
    @param  afile       input file
    @param  query_id    ID of query sequence (default='')
    Ensures: key = PDB accession
    @return {PDB-acc: [sequence_1, sequence_2, ...]}
    """

    seq_dict = {}
    header = ''
    seq = ''

    for aline in afile:
        aline = aline.strip()

        # check for header
        if aline.startswith('>'):
            if header != '' and seq != '':
                if seq_dict.has_key(header):
                    seq_dict[header].append(seq)
                else:
                    seq_dict[header] = [seq]
            seq = ''
            if aline.startswith('>%s' % query_id) and query_id !='':
                header = query_id
            else:
                header = aline[1:].split()[0]

        # otherwise concatenate sequence
        else:
            #aline_seq = aline.translate(None, '.-').upper()
            seq += aline

    # add last entry
    if header != '':
        if seq_dict.has_key(header):
            seq_dict[header].append(seq)
        else:
            seq_dict[header] = [seq]
    else:
        sys.stderr.write('ERROR: file empty or wrong file format')

    return seq_dict



if __name__ == "__main__":

    afile = open(sys.argv[1], 'r')
    if len(sys.argv) == 3:
        query_id = sys.argv[2]
    else:
        query_id = ''
    seq_dict = read_fasta(afile, query_id)
    afile.close()
    print 'There are %d entries with unique headers in your file.' % len(seq_dict)



import sys
sys.path.append("/home/mircomic/toolbox")
from parsing import parse_contacts

# command line input
infile_name = sys.argv[1]

# guessing separator of constraint file
test_line = open(infile_name,'r').readline()
if len(test_line.split(',')) != 1:
    sep = ','
elif len(test_line.split(' ')) != 1:
    sep = ' '
else:
    sep = '\t'

# parse constraint file
c_list = parse_contacts.parse(open(infile_name, 'r'))
    
# sort contacts and write simple string
for c in c_list:
    print ('%s %s %s' % (c[1], c[2], c[0]))



import sys

import parse_fasta
import parse_contacts

if __name__ == '__main__':

    idfile = open(sys.argv[1], 'r')
    score_dict = {}

    for line in idfile:
        id = line.strip()

        seqfile = open('data/%s:A/sequence.fa' % id, 'r')
        #contactfile = open('data/%s:A/mar23-hh0hh4hh10hhb40jh0jh4jh10jhm40-psicovplmdca.results' % id, 'r')
        contactfile = open('data/%s:A/pconse.results' % id, 'r')

        seq = parse_fasta.read_fasta(seqfile).values()[0][0]

        ref_len = len(seq)
        contacts = parse_contacts.parse(contactfile, ' ')
        if contacts == []:
            continue
        score_lst = []
        i = 0
        for contact in contacts:
            score_lst.append(contact[0])
            i += 1
            if i >= ref_len * 1.0:
                break
        avg_score = sum(score_lst) / float(len(score_lst))
        score_dict[id] = avg_score

    idfile.close()
    
    id_score_lst = sorted(score_dict.iteritems(), key=lambda(x):x[0])
    outfile = open(sys.argv[2], 'w')
    for (id, score) in id_score_lst:
        outfile.write('%s\t%s\n' % (id, score))
    outfile.close()



import sys
import operator
import numpy as np
from collections import defaultdict


def parse_atm_record(line):

    record = defaultdict()
    record['name'] = line[0:6].strip()
    record['atm_no'] = int(line[6:11])
    record['atm_name'] = line[12:16].strip()
    record['res_name'] = line[17:20].strip()
    record['chain'] = line[21]
    record['res_no'] = int(line[22:26])
    record['x'] = float(line[30:38])
    record['y'] = float(line[38:46])
    record['z'] = float(line[46:54])
    record['occ'] = float(line[54:60])
    record['B'] = float(line[60:66])
    
    return record


def read(pdbfile):

    header = ''
    res_lst = []
    atm_lst = []
    tail = ''

    seen_atoms = False
    curr_resi = 0
    prev_resi = 0
    
    for line in pdbfile:
        if not line.startswith('ATOM') and not seen_atoms:
            header += line
        elif not line.startswith('ATOM') and seen_atoms:
            tail += line
        else:
            atm_record = parse_atm_record(line)
            if not seen_atoms:
                curr_resi = atm_record['res_no']
                prev_resi = curr_resi
            seen_atoms = True
            curr_resi = atm_record['res_no']
            if curr_resi == prev_resi:
                atm_lst.append(line)
            else:
                #atm_lst.append(line)
                res_lst.append(atm_lst)
                atm_lst = [line]
            prev_resi = curr_resi
    res_lst.append(atm_lst)
     
    pdbfile.close()
    pdb_lst = [header, res_lst, tail]
    return pdb_lst


def write(pdb_lst, outfile):

    outfile.write(pdb_lst[0])

    for res in pdb_lst[1]:
        for atm in res:
            outfile.write(atm)
            
    outfile.write(pdb_lst[2])
    outfile.close()


def get_coordinates(pdbfile, chain):

    res_dict = defaultdict(list)

    if not chain:
        chain = get_first_chain(pdbfile)
        pdbfile.seek(0)

    for line in pdbfile:
        if not line.startswith('ATOM'):
            continue
        atm_record = parse_atm_record(line)
        if atm_record['chain'] != ' ' and atm_record['chain'] != chain:
            continue

        res_i = atm_record['res_no']
        atm = [atm_record['x'], atm_record['y'], atm_record['z']]

        """
        line_arr = line.split()
        #print line_arr

        if line_arr[2].startswith('H'):
            continue

        if len(line_arr[2]) > 4:
            if line_arr[3] != chain:
                continue
            try:
                res_i = int(line_arr[4])
            except ValueError as exc:
                continue
            try:
                atm = map(float, line_arr[5:8])
            except ValueError as exc:
                atm = [float('inf'), float('inf'), float('inf')]
        else:
            if line_arr[4] != chain:
                continue
            try:
                res_i = int(line_arr[5])
            except ValueError as exc:
                continue
            try:
                atm = map(float, line_arr[6:9])
            except ValueError as exc:
                atm = [float('inf'), float('inf'), float('inf')]
        """

        res_dict[res_i].append(np.array(atm))
        
    pdbfile.close()
    return sorted(res_dict.iteritems(), key=operator.itemgetter(0))


def get_cb_coordinates(pdbfile, chain):

    cb_lst = []
    res_dict = defaultdict(list)

    if not chain:
        chain = get_first_chain(pdbfile)
        pdbfile.seek(0)

    for line in pdbfile:
        if not line.startswith('ATOM'):
            continue
        atm_record = parse_atm_record(line)

        if atm_record['chain'] != ' ' and atm_record['chain'] != chain:
            continue

        res_i = atm_record['res_no']
        
        atm = [float('inf'), float('inf'), float('inf')]
        if 'GLY' in atm_record['res_name'] and atm_record['atm_name'] == 'CA':
                atm = [atm_record['x'], atm_record['y'], atm_record['z']]
                res_dict[res_i].append(np.array(atm))    
        elif atm_record['atm_name'] == 'CB':
                atm = [atm_record['x'], atm_record['y'], atm_record['z']]
                res_dict[res_i].append(np.array(atm))    
        

        """
        line_arr = line.split()

        if line_arr[4] != chain:
            continue

        if line_arr[2] == 'CA':
            try:
                res_i = int(line_arr[5])
            except ValueError as exc:
                continue
            try:
                atm = map(float, line_arr[6:9])
            except ValueError as exc:
                atm = [float('inf'), float('inf'), float('inf')]
            res_dict[res_i].append(np.array(atm))
        else:
            if line_arr[2] != 'CB':
                continue
            try:
                res_i = int(line_arr[5])
            except ValueError as exc:
                continue
            if len(line_arr[3]) > 3 and line_arr[3].startswith('A'):
                try:
                    atm = map(float, line_arr[6:9])
                except ValueError as exc:
                    atm = [float('inf'), float('inf'), float('inf')]
                atm_count += 1
                cb_lst.append(np.array(atm))
                res_dict[res_i].append(np.array(atm))
            elif len(line_arr[3]) == 3:
                try:
                    atm = map(float, line_arr[6:9])
                except ValueError as exc:
                    atm = [float('inf'), float('inf'), float('inf')]
                atm_count += 1
                cb_lst.append(np.array(atm))
                res_dict[res_i].append(np.array(atm))
        #print line_arr[2]
        
        if line_arr[3] == 'GLY' or line_arr[3] == 'AGLY':
            if line_arr[2] != 'CA':
                continue
            try:
                res_i = int(line_arr[5])
            except ValueError as exc:
                continue
            try:
                atm = map(float, line_arr[6:9])
            except ValueError as exc:
                atm = [float('inf'), float('inf'), float('inf')]
            #atm = map(float, line_arr[6:9])
            atm_count += 1
            cb_lst.append(np.array(atm))
        """
 
    cb_lst = []
    for i in xrange(res_i):
        if len(res_dict[i+1]) > 1:
            #print res_dict[i+1]
            cb_lst.append(res_dict[i+1][1])
        elif len(res_dict[i+1]) == 1:
            #print res_dict[i+1]
            cb_lst.append(res_dict[i+1][0])
    #print atm_count 
    pdbfile.close()
    #print cb_lst
    return cb_lst


def get_atom_seq(pdbfile, chain):

    three_to_one = {'ARG':'R', 'HIS':'H', 'LYS':'K', 'ASP':'D', 'GLU':'E', 'SER':'S', 'THR':'T', 'ASN':'N', 'GLN':'Q', 'CYS':'C', 'GLY':'G', 'PRO':'P', 'ALA':'A', 'ILE':'I', 'LEU':'L', 'MET':'M', 'PHE':'F', 'TRP':'W', 'TYR':'Y', 'VAL':'V', 'UNK': 'X'}
    res_dict = {}
 
    if not chain:
        chain = get_first_chain(pdbfile)
        pdbfile.seek(0)

    res_name = ''
    for line in pdbfile:
        if not line.startswith('ATOM'):
            continue
        atm_record = parse_atm_record(line)
        if atm_record['chain'] != ' ' and atm_record['chain'] != chain:
            continue

        res_i = atm_record['res_no']
        #print atm_record['res_name']
        if atm_record['res_name'] in three_to_one:
            #res_name = three_to_one[atm_record['res_name']]
            #print res_name
            res_name = three_to_one[atm_record['res_name']]
        #else:
            #res_name = ''
            #continue

        res_dict[res_i] = res_name

    res_lst = sorted(res_dict.iteritems(), key=operator.itemgetter(0))
    atom_seq = ''

    for res in res_lst:
        atom_seq += res[1]

    pdbfile.close()
    return atom_seq


def get_first_chain(pdbfile):

    for line in pdbfile:
        if not line.startswith('ATOM'):
            continue
        atm_record = parse_atm_record(line)
        break

    return atm_record['chain']
 



if __name__ == '__main__':

    pdbfile = open(sys.argv[1], 'r')
    chain = sys.argv[2]
    #print get_atom_seq(pdbfile, chain)
    pdbfile.close()
    pdbfile = open(sys.argv[1], 'r')
    #print get_coordinates(pdbfile)
    pdbfile.close()



import sys
import math

import parse_contacts
import parse_fasta


def get_min_dist(contact, contact_lst):

    min_dist = float('inf')

    x0 = contact[1]
    y0 = contact[2]

    for curr_contact in contact_lst:
        x1 = curr_contact[1]
        y1 = curr_contact[2]
        if x0 == x1 and y0 == y1:
            continue
        dist = math.sqrt(math.pow(x1-x0, 2) + math.pow(y1-y0, 2))
        if dist < min_dist:
            min_dist = dist
            min_x = x1
            min_y = y1

    #print '(%d,%d) to (%d,%d): %d' % (x0, y0, min_x, min_y, min_dist)
    
    return min_dist


def main(seqfile, contactfile, sep=' '):
    
    seq = parse_fasta.read_fasta(seqfile).values()[0][0]
    seq_len = len(seq)

    all_contacts = parse_contacts.parse(contactfile, sep)
    top_contacts = []
    for i, contact in enumerate(all_contacts):
        if abs(contact[1] - contact[2]) < 10:
            continue
        top_contacts.append(contact)
        if i >= seq_len:
            break
    min_dist_lst = []
    for contact in top_contacts:
        min_dist = get_min_dist(contact, top_contacts)
        min_dist_lst.append(min_dist)
    #return sum(min_dist_lst) / float(len(min_dist_lst))
    return min_dist_lst



if __name__ == '__main__':
    
    idfile = open(sys.argv[1], 'r')
    dist_dict = {}
    for line in idfile:
        id = line.strip().split()
        if len(id) == 1:
            id = id[0]
            seqfile = open('data/%s:A/sequence.fa' % id, 'r')
            contactfile = open('data/%s:A/pconse.results' % id, 'r')
            dist_dict[id] = main(seqfile, contactfile)
        else:
            seqfile = open('pfam/data/pconsc_predictions/%s/%s.fa' % (id[0], id[1]), 'r')
            contactfile = open('pfam/data/pconsc_predictions/%s/%s.out' % (id[0], id[1]), 'r')
            dist_dict[id[0]] = main(seqfile, contactfile)
        seqfile.close()
        contactfile.close()
        #contactfile = open('data/%s:A/mar23-hh0hh4hh10hhb40jh0jh4jh10jhm40-psicovplmdca.results' % id, 'r')


    idfile.close()

    id_dist_lst = sorted(dist_dict.iteritems(), key=lambda(x):x[0])
    #print id_dist_lst
    outfile = open(sys.argv[2], 'w')
    for (id, dist_lst) in id_dist_lst:
        outfile.write('%s\t%s\n' % (id, '\t'.join(map(str, dist_lst))))
    outfile.close()






from distutils.core import setup

setup(
    name='ContactVis',
    version='0.1.1',
    author='M. Michel',
    author_email='guy.inkognito42@gmail.com',
    packages=['contactvis', 'contactvis.parsing', 'contactvis.test'],
    url='https://github.com/MMichel/contact-vis.git',
    license='LICENSE.txt',
    description='Contact map plotting for predicted protein residue-residue contacts.',
    long_description=open('README.md').read(),
    install_requires=[
        "Biopython >= 1.59",
        "matplotlib >= 1.3.1",
    ],
)
